from dataclasses import dataclass
from typing import Any, Optional

import pytest

from ..colors import true_color_supported


@dataclass
class ColorTestCase:
    func_name: str
    message: Any = ''
    color: Optional[bool] = None
    expected: str = ''


TRUE_COLOR = true_color_supported()


@pytest.mark.parametrize('test_case', [
    # standard uses
    ColorTestCase('black', message='hi', color=True, expected='\x1b[30mhi\x1b[0m' if not TRUE_COLOR else f'\x1b[38;2;0;0;0mhi\x1b[0m'),
    ColorTestCase('blue', message='hi', color=True, expected='\x1b[34mhi\x1b[0m' if not TRUE_COLOR else f'\x1b[38;2;0;0;255mhi\x1b[0m'),
    ColorTestCase('cyan', message='hi', color=True, expected='\x1b[36mhi\x1b[0m' if not TRUE_COLOR else f'\x1b[38;2;0;255;255mhi\x1b[0m'),
    ColorTestCase('green', message='hi', color=True, expected='\x1b[32mhi\x1b[0m' if not TRUE_COLOR else f'\x1b[38;2;0;255;0mhi\x1b[0m'),
    ColorTestCase('grey', message='hi', color=True, expected='\x1b[37m\x1b[2mhi\x1b[0m' if not TRUE_COLOR else f'\x1b[38;2;127;127;127mhi\x1b[0m'),
    ColorTestCase('magenta', message='hi', color=True, expected='\x1b[35mhi\x1b[0m' if not TRUE_COLOR else f'\x1b[38;2;255;0;255mhi\x1b[0m'),
    ColorTestCase('red', message='hi', color=True, expected='\x1b[31mhi\x1b[0m' if not TRUE_COLOR else f'\x1b[38;2;255;0;0mhi\x1b[0m'),
    ColorTestCase('yellow', message='hi', color=True, expected='\x1b[33mhi\x1b[0m' if not TRUE_COLOR else f'\x1b[38;2;255;255;0mhi\x1b[0m'),
    ColorTestCase('white', message='hi', color=True, expected='\x1b[37mhi\x1b[0m' if not TRUE_COLOR else f'\x1b[38;2;255;255;255mhi\x1b[0m'),

    # test uncolored uses
    ColorTestCase('black', message='hi', color=False, expected='hi'),
    ColorTestCase('blue', message='hi', color=False, expected='hi'),
    ColorTestCase('cyan', message='hi', color=False, expected='hi'),
    ColorTestCase('green', message='hi', color=False, expected='hi'),
    ColorTestCase('grey', message='hi', color=False, expected='hi'),
    ColorTestCase('magenta', message='hi', color=False, expected='hi'),
    ColorTestCase('red', message='hi', color=False, expected='hi'),
    ColorTestCase('yellow', message='hi', color=False, expected='hi'),
    ColorTestCase('white', message='hi', color=False, expected='hi'),
    ])
def test_colors(test_case):
    from .. import colors

    func = getattr(colors, test_case.func_name)
    result = func(message=test_case.message, color=test_case.color, truecolor=TRUE_COLOR)
    assert result == test_case.expected, f'TrueColor={TRUE_COLOR} ({tuple(result)}) != ({tuple(test_case.expected)})'


@dataclass
class RgbColorTestCase:
    red: int = 0
    green: int = 0
    blue: int = 0
    message: Any = ''
    color: Optional[bool] = None


@pytest.mark.parametrize('test_case', [
    # black
    RgbColorTestCase(red=0, green=0, blue=0, message='hi', color=True),
    # blue
    RgbColorTestCase(red=0, green=0, blue=255, message='hi', color=True),
    # cyan
    RgbColorTestCase(red=0, green=255, blue=255, message='hi', color=True),
    # green
    RgbColorTestCase(red=0, green=255, blue=0, message='hi', color=True),
    # grey
    RgbColorTestCase(red=127, green=127, blue=127, message='hi', color=True),
    # magenta
    RgbColorTestCase(red=255, green=0, blue=255, message='hi', color=True),
    # red
    RgbColorTestCase(red=255, green=0, blue=0, message='hi', color=True),
    # yellow
    RgbColorTestCase(red=255, green=255, blue=0, message='hi', color=True),
    # white
    RgbColorTestCase(red=255, green=255, blue=255, message='hi', color=True),

    # black
    RgbColorTestCase(red=0, green=0, blue=0, message='hi', color=False),
    # blue
    RgbColorTestCase(red=0, green=0, blue=255, message='hi', color=False),
    # cyan
    RgbColorTestCase(red=0, green=255, blue=255, message='hi', color=False),
    # green
    RgbColorTestCase(red=0, green=255, blue=0, message='hi', color=False),
    # grey
    RgbColorTestCase(red=127, green=127, blue=127, message='hi', color=False),
    # magenta
    RgbColorTestCase(red=255, green=0, blue=255, message='hi', color=False),
    # red
    RgbColorTestCase(red=255, green=0, blue=0, message='hi', color=False),
    # yellow
    RgbColorTestCase(red=255, green=255, blue=0, message='hi', color=False),
    # white
    RgbColorTestCase(red=255, green=255, blue=255, message='hi', color=False),
    ])
def test_colors_rgb(test_case):
    from .. import colors

    prefix = f'\x1b[38;2;{test_case.red};{test_case.green};{test_case.blue}m'
    reset = f'\x1b[0m'
    expected = f'{prefix}{test_case.message}{reset}' if test_case.color else f'{test_case.message}'

    result = colors.rgb(red=test_case.red, blue=test_case.blue, green=test_case.green, message=test_case.message, color=test_case.color, truecolor=True)
    assert result == expected


def test_colors_factory():
    from .. import colors

    all_colors = {
        'BLACK': colors.BLACK, 'RED': colors.RED, 'GREEN': colors.GREEN,
        'BLUE': colors.BLUE, 'CYAN': colors.CYAN, 'MAGENTA': colors.MAGENTA,
        'YELLOW': colors.YELLOW, 'WHITE': colors.WHITE, 'GREY': colors.GREY
        }

    new_colors = {
        'BLACK': [colors.Color(), colors.Color(red=0, green=0, blue=0)],
        'RED': [colors.Color(red=255), colors.Color(red=255, green=0, blue=0)],
        'YELLOW': [colors.Color(red=255, green=255), colors.Color(red=255, green=255, blue=0)],
        'MAGENTA': [colors.Color(red=255, blue=255), colors.Color(red=255, green=0, blue=255)],
        'CYAN': [colors.Color(green=255, blue=255), colors.Color(red=0, green=255, blue=255)],
        'BLUE': [colors.Color(blue=255), colors.Color(red=0, green=0, blue=255)],
        'GREEN': [colors.Color(green=255), colors.Color(red=0, green=255, blue=0)],
        'WHITE': [colors.Color(red=255, green=255, blue=255)],
        'GREY': [colors.Color(red=127, green=127, blue=127)],
        }

    for index, color in enumerate(all_colors.values()):
        previous = list(all_colors.values())[index - 1]
        assert color != previous
        assert color.keys() == ('red', 'green', 'blue')

    for name, values in new_colors.items():
        original = all_colors[name]
        assert all([original == value for value in values]), f'{name}: {original} not equal to {values}'

    color = colors.Color()
    assert len(color) == 3
    assert list(color) == [0, 0, 0]
    assert list(color.keys()) == ['red', 'green', 'blue']
    for key in color.keys():
        assert color[key] == getattr(color, key)
    with pytest.raises(KeyError):
        assert color['not-a-key']



import functools
from dataclasses import dataclass, field
from typing import Any, Dict, List


@dataclass
class factory:
    names: List[str] = field(default_factory=list)
    repository: Dict[Any, Any] = field(default_factory=dict)

    def __call__(self, cls, *args, **kwds):

        @functools.wraps(cls.__new__)
        def new(cls, *args, **kwds):
            cls_key = {}
            for index, name in enumerate(self.names):
                if name in kwds:
                    value = kwds.get(name)
                elif len(args) > index:
                    value = args[index]
                elif hasattr(cls, name):
                    value = getattr(cls, name)
                else:
                    # Guard for programming errors during development
                    raise KeyError(f'Could not find `{name}` in {cls} `__new__` invocation: args={args} kwds={kwds}')
                cls_key[name] = value
            vals = tuple(cls_key.values())
            if vals not in self.repository:
                instance = object.__new__(cls)
                self.repository[vals] = instance
            instance = self.repository[vals]
            return instance

        cls.__new__ = new

        return cls



"""This is imported by setup.py and contains all of the information
necessary to run setup.py
"""
import os
import subprocess
from dataclasses import asdict, dataclass, field
from pathlib import Path
from typing import Dict, Tuple


# ----------------------------------------------------------------------
# Package Metadata
# ----------------------------------------------------------------------
@dataclass
class PackageMetadata:
    """Package metadata information

    Notes:
        This is used in setup, scripts, cli interface, endpoints, etc.

    Attributes:
        name: name of package
        package_name: import name for package
        title: short human readable title for package
        summary: long human readable title for package
        description: long description for package

        version: semantic version of package
        major: major component of semantic version (X._._)
        minor: minor component of semantic version (_.X._)
        micro: micro component of semantic version (_._.X)
        build: git sha value of build
        release: human friendly release name

        author: package author
        author_email: email for package author

        maintainer: package maintainer
        maintainer_email: package maintainer email

        copyright: copyright years of major work on package
        license: default license for package

        url: location for package repository or documentation

        classifiers:  python trove classifiers
        keywords: keywords for searching for package

    Example:
        The PackageMetadata attributes can be accessed by something
        like the following.

        >>> from termlog.__metadata__ import package_metadata
        >>> package_metadata.name
        termlog

    """
    name: str = 'termlog'
    version: str = '1.0.2'
    package_name: str = field(init=False, repr=False, default='')
    title: str = f'A terminal logging library'
    summary: str = 'Create beautiful terminal structured '\
                   'output for developers and production systems'

    major: int = field(init=False, repr=False)
    minor: int = field(init=False, repr=False)
    micro: int = field(init=False, repr=False)
    build: str = field(init=False, repr=False)
    release: str = field(init=False, repr=False)

    description: str = f'''{name} v{version}: {title}'''
    author: str = 'Brian Bruggeman'
    author_email: str = 'brian.m.bruggeman@gmail.com'

    maintainer: str = 'Brian Bruggeman'
    maintainer_email: str = 'brian.m.bruggeman@gmail.com'

    copyright: str = f'2019'
    license: str = f'MIT'

    url: str = 'https://github.com/brianbruggeman/termlog'
    bug_reports_url: str = 'https://github.com/brianbruggeman/termlog/issues'
    documentation_url: str = 'https://termlog.readthedocs.io/en/latest/'
    travis_CI_url: str = 'https://travis-ci.org/brianbruggeman/termlog'
    code_coverage_url: str = 'https://codecov.io/gh/brianbruggeman/termlog'
    project_urls: Dict = field(init=False, repr=False, default_factory=dict)

    classifiers: Tuple = (
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: MacOS',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: System :: Logging',
        'Topic :: Terminals',
        'Typing :: Typed',
        )

    keywords: Tuple = (
        'terminal',
        'logger',
        'docker',
        )

    @property
    def setup(self) -> Dict:
        """Returns setup.py friendly fields and values"""
        # see: https://packaging.python.org/specifications/core-metadata/
        required_fields = [
            'metadata_version', 'name', 'version',
            ]
        setup_friendly_fields = required_fields + [
            'platform', 'supported_platform', 'description',
            'description_content_type', 'keywords', 'home_page', 'download_url',
            'author', 'author_email', 'maintainer', 'maintainer_email',
            'license', 'classifier', 'requires_dist', 'requires_python',
            'requires_external', 'project_urls', 'provides_extra',
            'provides_dist', 'obsoletes_dist',

            'long_description', 'long_description_content_type',
            'license', 'version', 'author', 'author_email',
            'maintainer', 'maintainer_email', 'license', 'url',
            'classifiers', 'keywords'
            ]
        data = {}
        for key, value in asdict(self).items():
            if key in setup_friendly_fields:
                data[key] = value
        data['keywords'] = list(data['keywords'])
        data['classifiers'] = list(data['classifiers'])
        return data

    def __post_init__(self):
        self.package_name = self.name.replace('-', '_')
        self.major, self.minor, self.micro = list(map(int, self.version.split('.')))
        self.build = self._get_build() or ''
        if self.build:
            self.release = f'{self.name} {self.version} [build: {self.build}]'
        else:
            self.release = f'{self.name} {self.version}'
        for key, value in asdict(self).items():
            if key.endswith('_url'):
                name = ' '.join(key[:-4].split('_')).capitalize()
                self.project_urls.setdefault(name, value)

    def _get_build(self):
        # localhost
        git_sha = self._get_git_sha()
        if git_sha:
            return git_sha

        # AWS
        filepath_sha = self._get_filepath_git_sha()
        if filepath_sha:
            return filepath_sha

        # docker
        try:
            # In an effort to clearly identify where we use environment
            # variables, this will use config, but it creates both
            # a circular dependency and causes problems for setup.py
            # which reads this file.  So punt if there are problems
            git_sha_env = os.getenv('CDR_REST_GIT_SHA')
            if git_sha_env:
                return git_sha_env
        except ImportError:
            # Otherwise this will fail during setup.py
            print('Failed to capture build')
            for key, value in sorted(os.environ.items()):
                print(f'{key}: {value}')

    @staticmethod
    def _get_git_sha():
        # In a development enviromment this will have access to git
        #  and it's possible to simply query the last hash value
        git_command = ('git', 'rev-parse', '--short', 'HEAD')
        proc = subprocess.run(git_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if proc.returncode == 0:
            return proc.stdout.decode('utf-8').strip('\n')

    @staticmethod
    def _get_filepath_git_sha():
        # In a production environment, the hash value should be
        #  part of the file path
        for path_fragment in Path(__file__).absolute().parts:
            if not path_fragment or path_fragment == '/':
                continue
            if path_fragment.startswith('termlog-'):
                if 'git.sha' in path_fragment:
                    return path_fragment.split('termlog-')[-1]

    def keys(self):
        yield from self.__annotations__.keys()

    def __getitem__(self, item):
        if item in self.__annotations__.keys():
            value = getattr(self, item)
            return value
        raise AttributeError(f"Could not find {item}")

    def __iter__(self):
        for field in self.__annotations__.keys():
            yield field


package_metadata = PackageMetadata()



# flake8: noqa
from .__metadata__ import package_metadata   # isort:skip

__author__ = package_metadata.author
__version__ = package_metadata.version


from .colors import *
from .formatting import *
from .terminal import *




"""Color module

Contains resources to build 256 (standard) and 16m (true-color) text
output.

"""
import os
from dataclasses import asdict, dataclass
from typing import Any, Dict, Optional, Tuple

from .decorations import factory

__all__ = (
    'black', 'blue', 'cyan', 'green', 'grey', 'magenta', 'rgb', 'red',
    'yellow', 'white', 'Color'
    )


_colors: Dict[Tuple[int, int, int], 'Color'] = {}


# This will get updated below, see: _true_color_supported
def true_color_supported() -> bool:
    """Check if truecolor is supported by the current tty.

    Note: this currently only checks to see if COLORTERM contains
          one of the following enumerated case-sensitive values:
             - truecolor
             - 24bit

    """
    color_term = os.getenv('COLORTERM', '')
    return True if any(check in color_term for check in ['truecolor', '24bit']) else False


TRUE_COLOR_SUPPORTED = true_color_supported()
ESCAPE: str = '\x1b'

# style codes
#  Not all of these are supported within all terminals
BRIGHT: str = f'{ESCAPE}[1m'
DIM: str = f'{ESCAPE}[2m'
ITALIC: str = f'{ESCAPE}[3m'
UNDERLINED: str = f'{ESCAPE}[4m'
BLINK: str = f'{ESCAPE}[5m'
STROBE: str = f'{ESCAPE}[6m'
INVERTED: str = f'{ESCAPE}[7m'
HIDDEN: str = f'{ESCAPE}[8m'


@factory(names=['red', 'green', 'blue'], repository=_colors)
@dataclass
class Color:
    """A data structure for packaging up Color display information

    Example:
        >>> from termlog import Color, echo
        >>> solarized_red = Color(red=220, green=50, blue=47)
        >>> solarized_magenta = Color(red=221, green=54, blue=130)
        >>> msg = ...
        >>> echo(f'{solarized_red("ERROR")}: {solarized_magenta(msg)}')

    Attributes:
        red: the value for the red aspect of the color [0-255]
        green: the value for the green aspect of the color [0-255]
        blue: the value for the blue aspect of the color [0-255]
        color_prefix: the string prefix when true-color is not enabled
        true_color_prefix: the string prefix for when true-color is enabled
        suffix: the reset suffix
        truecolor: enables true color when True
        bright: a bold/bright version of the color
        dim: a dim version of the color
        italic: an italic version of the color
        underlined: an underlined version of the message

    """
    red: int = 0
    green: int = 0
    blue: int = 0
    color_prefix: str = ''
    true_color_prefix: str = f'{ESCAPE}[38;2;{{red}};{{green}};{{blue}}m'
    suffix: str = f'{ESCAPE}[0m'
    truecolor: bool = TRUE_COLOR_SUPPORTED

    # style attributes
    dim: bool = False
    bright: bool = False
    italic: bool = False
    underline: bool = False
    blink: bool = False
    inverted: bool = False
    hidden: bool = False
    strobe: bool = False

    def __post_init__(self):
        # cap red, green and blue
        prefix_mapping = {
            (0, 0, 0): 30,  # black
            (255, 0, 0): 31,  # red
            (0, 255, 0): 32,  # green
            (255, 255, 0): 33,  # yellow
            (0, 0, 255): 34,  # blue
            (255, 0, 255): 35,  # magenta
            (0, 255, 255): 36,  # cyan
            (255, 255, 255): 37,  # white
            (127, 127, 127): 37,  # grey
            }
        if self.color_prefix == '':
            key = self.red, self.green, self.blue
            mapped = prefix_mapping.get(key)
            self.color_prefix = f'{ESCAPE}[{mapped}m' or ''
            if key == (127, 127, 127) and self.color_prefix:
                self.color_prefix += DIM
        if not self.color_prefix:
            self.color_prefix = self.true_color_prefix
        self.red = max(min(int(self.red or 0), 255), 0)
        self.green = max(min(int(self.green or 0), 255), 0)
        self.blue = max(min(int(self.blue or 0), 255), 0)

    def __call__(self, message: Any, color: bool = True, truecolor: Optional[bool] = None) -> str:
        truecolor = self.truecolor if truecolor is None else truecolor
        if color:
            if truecolor:
                prefix = self.true_color_prefix.format(**asdict(self))
            else:
                prefix = self.color_prefix
            if self.hidden:
                prefix += HIDDEN
            else:
                if self.dim:
                    prefix += DIM
                elif self.bright:
                    prefix += BRIGHT
                if self.italic:
                    prefix += ITALIC
                if self.underline:
                    prefix += UNDERLINED
                if self.blink:
                    prefix += BLINK
                if self.strobe:
                    prefix += STROBE
            return f'{prefix}{message}{self.suffix}'
        else:
            return f'{message}'

    def __eq__(self, other):
        return id(self) == id(other)

    def __getitem__(self, item):
        if item in ('red', 'green', 'blue'):
            return getattr(self, item)
        else:
            raise KeyError(f'Could not find {item} in {self}')

    @staticmethod
    def keys():
        return ('red', 'green', 'blue')

    @staticmethod
    def __len__():
        return 3

    def __iter__(self):
        for name in ('red', 'green', 'blue'):
            yield getattr(self, name)


# Set simple terminal colors
BLACK = Color(0, 0, 0, color_prefix=f'{ESCAPE}[30m')
RED = Color(255, 0, 0, color_prefix=f'{ESCAPE}[31m')
GREEN = Color(0, 255, 0, color_prefix=f'{ESCAPE}[32m')
YELLOW = Color(255, 255, 0, color_prefix=f'{ESCAPE}[33m')
BLUE = Color(0, 0, 255, color_prefix=f'{ESCAPE}[34m')
MAGENTA = Color(255, 0, 255, color_prefix=f'{ESCAPE}[35m')
CYAN = Color(0, 255, 255, color_prefix=f'{ESCAPE}[36m')
WHITE = Color(255, 255, 255, color_prefix=f'{ESCAPE}[37m')
GREY = Color(127, 127, 127, color_prefix=f'{ESCAPE}[37m{ESCAPE}[2m')


def black(message: Any, color: bool = True, truecolor: Optional[bool] = None) -> str:
    """Add black color to *message*.

    Example:
        >>> from termlog import black, echo
        >>> f'{black("black")}'
        '{ESCAPE}[38;2;0;0;0mblack{ESCAPE}[0m'

    Args:
        message: text to color
        color: enable color on output

    Returns:
        Colored text if color is enabled

    """
    return rgb(message=message, color=color, truecolor=truecolor)


def blue(message: Any, color: bool = True, truecolor: Optional[bool] = None) -> str:
    """Add blue color to *message*.

    Example:
        >>> from termlog import black, echo
        >>> echo(f'A {blue("blue")} message')
        '20190606105532 a {ESCAPE}[38;2;0;0;255mblue{ESCAPE}[0m message'

    Args:
        message: text to color
        color: enable color on output

    Returns:
        Colored text if color is enabled

    """
    return rgb(message=message, color=color, blue=255, truecolor=truecolor)


def cyan(message: Any, color: bool = True, truecolor: Optional[bool] = None) -> str:
    """Add cyan color to *message*.

    Example:
        >>> from termlog import black, echo, set_config
        >>> set_config(timestamp=False)
        >>> echo(f'A {cyan("cyan")} message')
        'a {ESCAPE}[38;2;0;255;255mcyan{ESCAPE}[0m message'

    Args:
        message: text to color
        color: enable color on output

    Returns:
        Colored text if color is enabled

    """
    return rgb(message=message, color=color, green=255, blue=255, truecolor=truecolor)


def green(message: Any, color: bool = True, truecolor: Optional[bool] = None) -> str:
    """Add green color to *message*.

    Example:
        >>> from termlog import echo, green
        >>> msg = green("green")
        >>> print(msg)
        '{ESCAPE}[38;2;0;255;0mgreen{ESCAPE}[0m'
        >>> echo(f'A {msg} message', color=False)
        '20190606143659 a green message'

    Args:
        message: text to color
        color: enable color on output

    Returns:
        Colored text if color is enabled

    """
    return rgb(message=message, color=color, green=255, truecolor=truecolor)


def grey(message: Any, color: bool = True, truecolor: Optional[bool] = None) -> str:
    """Add grey color to *message*.

    Example:
        >>> from termlog import echo, grey
        >>> echo(f'A {grey("grey")} message', json=True)
        '{"data": "a \u001b[38;2;127;127;127mgrey\u001b[0m message", "timestamp": "20190606144743"}'

    Args:
        message: text to color
        color: enable color on output

    Returns:
        Colored text if color is enabled

    """
    return rgb(message=message, color=color, red=127, green=127, blue=127, truecolor=truecolor)


def magenta(message: Any, color: bool = True, truecolor: Optional[bool] = None) -> str:
    """Add orange color to *message*.

    Example:
        >>> from termlog import echo, magenta
        >>> msg = magenta('magenta')
        >>> echo(f'A {msg} message', json=True)
        '20190606143659 a {ESCAPE}[38;2;255;0;255mmagenta{ESCAPE}[0m message'

    Args:
        message: text to color
        color: enable color on output

    Returns:
        Colored text if color is enabled

    """
    return rgb(message=message, color=color, red=255, blue=255, truecolor=truecolor)


def rgb(message: Any, red: int = 0, green: int = 0, blue: int = 0, color: bool = True, truecolor: Optional[bool] = None) -> str:
    """Add any true color to *message*.

    Example:
        >>> from termlog import echo, rgb, Color
        >>> c = Color(red=223, green=81, blue=127)
        >>> print(f'{c!r}')
        Color(red=223, green=81, blue=127, color_prefix='', true_color_prefix='{ESCAPE}[38;2;{red};{green};{blue}m', suffix='{ESCAPE}[0m', truecolor=False)
        >>> f'A {rgb("colored", **c)} message'
        'A {ESCAPE}[38;2;223;81;127mcolored{ESCAPE}[0m message'

    Note: this only works if the terminal supports true color and there
          is no way to definitively determine if truecolor is supported
          by the terminal.

    Args:
        message: text to color
        red: red portion of color [0 to 255]
        green: green portion of color [0 to 255]
        blue: blue portion of color [0 to 255]
        color: enable color on output

    Returns:
        Colored text if color is enabled

    """
    truecolor = TRUE_COLOR_SUPPORTED if truecolor is None else truecolor
    _color = Color(red=red, green=green, blue=blue, truecolor=truecolor)
    return _color(message, color)


def red(message: Any, color: bool = True, truecolor: Optional[bool] = None) -> str:
    """Add red color to *message*.

    Example:
        >>> from termlog import echo, red
        >>> f'A {red("red")} message'
        'A {ESCAPE}[38;2;225;0;0mred{ESCAPE}[0m message'

    Args:
        message: text to color
        color: enable color on output

    Returns:
        Colored text if color is enabled

    """
    return rgb(message=message, color=color, red=255, truecolor=truecolor)


def yellow(message: Any, color: bool = True, truecolor: Optional[bool] = None) -> str:
    """Add yellow color to *message*.

    Example:
        >>> from termlog import echo, yellow
        >>> f'A {yellow("yellow")} message'
        'A {ESCAPE}[38;2;225;255;0myellow{ESCAPE}[0m message'

    Args:
        message: text to color
        color: enable color on output

    Returns:
        Colored text if color is enabled

    """
    return rgb(message=message, color=color, red=255, green=255, truecolor=truecolor)


def white(message: Any, color: bool = True, truecolor: Optional[bool] = None) -> str:
    """Add white color to *message*.

    Example:
        >>> from termlog import echo, white
        >>> f'A {white("white")} message'
        'A {ESCAPE}[38;2;225;255;255mwhite{ESCAPE}[0m message'

    Args:
        message: text to color
        color: enable color on output

    Returns:
        Colored text if color is enabled

    """
    return rgb(message=message, color=color, red=255, green=255, blue=255, truecolor=truecolor)



import sys
from dataclasses import dataclass
from typing import IO, Any, Optional, Union

from pygments.lexer import Lexer

from .formatting import format

__all__ = ('echo', 'set_config')


@dataclass
class TerminalConfig:
    """Data class to hold some simple configuration data

    Attributes:
        color: when True, display output with color
        json: when True, display output as json
        timestamp: when True, include timestamp in output
        time_format: control how the timestamp appears in the output

    """
    color: bool = True
    json: bool = False
    timestamp: Optional[bool] = None
    time_format: str = '%Y%m%d%H%M%S'


_terminal_config = TerminalConfig()


def set_config(
        color: Optional[bool] = None,
        json: Optional[bool] = None,
        time_format: Optional[str] = None,
        timestamp: Optional[bool] = None,
        ) -> TerminalConfig:
    global _terminal_config
    _terminal_config.color = color if color is not None else _terminal_config.color
    _terminal_config.json = json if json is not None else _terminal_config.json
    _terminal_config.timestamp = timestamp if timestamp is not None else _terminal_config.timestamp
    _terminal_config.time_format = time_format if time_format is not None else _terminal_config.time_format
    return _terminal_config


def echo(*messages: Any,
         verbose: Optional[Union[bool, int]] = None,
         end: str = '\n',
         flush: bool = True,
         lexer: Optional[Union[Lexer, str]] = None,
         file: IO = sys.stdout,
         color: Optional[bool] = None,
         json: Optional[bool] = None,
         time_format: Optional[str] = None,
         add_timestamp: Optional[bool] = None,
         ) -> Optional[str]:
    """Echo *message*.

    lexers can be passed in, and if they are, the resulting output
    will attempt to be setup

    Args:
        messages: messages to echo
        verbose: verbosity level
        end: text to print at end of message, defaults to newline
        flush: flush output after write
        lexer: message lexical analyzer
        file: file to echo message to
        json: Dump out a structured text
        time_format: string to control time formatting
        string_format: string to control string formatting
        add_timestamp: Controls whether the timestamp is dumped out
        color: control color output

    Returns:
        Tuple[str]: beautified messages

    """
    # verbose should be minimally capped to 0
    verbose = 1 if verbose is None else max(int(verbose or 0), 0)
    if not verbose:
        return None

    time_format = time_format if time_format is not None else _terminal_config.time_format
    add_timestamp = add_timestamp if add_timestamp is not None else _terminal_config.timestamp
    json = json if json is not None else _terminal_config.json
    color = color if color is not None else _terminal_config.color

    string = format(*messages, lexer=lexer, color=color, json=json, time_format=time_format, add_timestamp=add_timestamp)
    print(string, file=file, end=end, flush=flush)
    return string



import textwrap
from pathlib import Path
from typing import Any, Optional, Union

import pygments
from pygments.formatters import get_formatter_by_name
from pygments.lexer import Lexer
from pygments.lexers import get_lexer_by_name, guess_lexer

from .message import Message

__all__ = ('beautify', 'format')


def beautify(
        message: Any,
        indent: int = 0,
        lexer: Optional[Union[pygments.lexer.Lexer, str]] = None
        ) -> str:
    """Beautify *message*.

    Args:
        message: message to beautify
        indent: number of spaces to indent
        lexer: message lexical analyzer

    Returns:
        str: beautified message

    """
    formatter = get_formatter_by_name('16m')

    indent = max(int(indent or 0), 0)
    if isinstance(message, Path):
        message = str(message)
    elif isinstance(message, bytes):
        message = message.decode('utf-8')

    single = False
    if isinstance(message, str):
        single = len(message) - len(message.rstrip('\n')) == 0

    if isinstance(message, Exception):
        lexer = lexer or 'py3tb'

    if isinstance(lexer, str):
        lexer = get_lexer_by_name(lexer)

    if not lexer and isinstance(message, str):
        lexer = guess_lexer(message, stripall=True) if lexer is None else lexer

    if lexer and isinstance(message, str):
        messages = message.split('\n')
        for index, message in enumerate(messages):
            message = pygments.highlight(message, lexer, formatter)
            messages[index] = message.strip()
        message = '\n'.join(messages)
    if indent > 0:
        message = textwrap.indent(message, ' ' * indent)
    return message.rstrip('\n') if single and isinstance(message, str) else message


def format(*messages: Any,
           lexer: Optional[Union[Lexer, str]] = None,
           color: bool = None,
           json: bool = None,
           time_format: Optional[str] = None,
           add_timestamp: Optional[bool] = None,
           ) -> str:
    """Echo *message*.

    lexers can be passed in, and if they are, the resulting output
    will attempt to be setup

    Args:
        messages: messages to echo
        color: colorize output
        lexer: message lexical analyzer
        json: Dump out a structured text
        time_format: control time format
        add_timestamp: add a timestamp to the output

    Returns:
        Tuple[str]: beautified messages

    """
    # Allows echo to be used in settings and prevents circular dependencies
    string = ''
    for index, message in enumerate(messages):
        include_timestamp = False
        if add_timestamp is True:
            include_timestamp = True
        if add_timestamp is None:
            if index == 0:
                include_timestamp = True
            elif json is True:
                include_timestamp = True
        msg = Message(
            data=message,
            lexer=str(lexer) if lexer else '',
            json=bool(json),
            color=bool(color),
            time_format=time_format,
            include_timestamp=include_timestamp,
            )
        separator = ('\n' if json else ' ') if string else ''
        string = f'{string}{separator}{msg}'
    return string



#!/usr/bin/env python
# isort:skip
"""
Setup.py shouldn't be updated during normal development.

Update:
   entrypoints.txt  <-- add new command-line interfaces here
   requirements.txt  <-- required for package to run
   requirements/*.txt  <-- as appropriate
"""
# ----------------------------------------------------------------------
# Version guard
# ----------------------------------------------------------------------
import sys
if sys.version_info < (3, 7):
    sys.stderr.write('Python 3.7+ is required for installation.\n')
    sys.tracebacklimit = 0
    sys.exit(1)

# ----------------------------------------------------------------------
import datetime
import itertools
import os
import re
import subprocess

# ----------------------------------------------------------------------
from pathlib import Path
from typing import List, Optional, Tuple

from setuptools import Command, setup



try:
    # pip10+
    import pip._internal.req as req
except ImportError:
    # pip9
    import pip.req as req


try:
    import pypandoc
except ImportError:
    pypandoc = None


# ----------------------------------------------------------------------
def main():
    """Run setup"""
    top_path = str(Path(__file__).parent.absolute())
    metadata = get_package_metadata(top_path=top_path)

    # Run setup
    setup(**metadata)


# ----------------------------------------------------------------------
# Commands
# ----------------------------------------------------------------------
class BuildImageCommand(Command):
    description: str = 'Builds a docker image'
    user_options: List = list()

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from scripts.build import build_image
        build_image()


class CleanCommand(Command, object):
    description: str = 'Remove build artifacts and *.pyc'
    user_options: List = [
        ('force', 'f', 'Force clean')
        ]

    def initialize_options(self):
        self.force: bool = False

    def finalize_options(self):
        pass

    def run(self):
        self.force = True if self.force else False
        from scripts.clean import clean
        clean(force=self.force)


class CheckCommand(Command, object):
    description: str = 'Check build using twine check'
    user_options: List = [
        ]

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from scripts.check import check
        check()


class DistCommand(Command, object):
    description = 'Builds sdist and bdist_wheel for release'
    user_options: List = list()

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from scripts.build import build_dist
        build_dist()


class LintCommand(Command, object):
    description = 'Runs linting and style tools'
    user_options: List = list()

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from scripts.lint import lint
        lint()


class PrereleaseCommand(Command, object):
    description = 'Runs tests'
    user_options: List = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from scripts.build import build_dist
        from scripts.clean import clean
        from scripts.check import check
        from scripts.lint import lint
        from scripts.test import run_test
        from scripts.push import push_pypi

        clean(force=True)
        build_dist()
        lint()
        check()
        run_test(release=True, coverage_fail_threshold=75)


class ReleaseCommand(Command, object):
    description = 'Runs tests'
    user_options: List = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from scripts.build import build_dist
        from scripts.clean import clean
        from scripts.check import check
        from scripts.lint import lint
        from scripts.test import run_test
        from scripts.push import push_pypi

        clean(force=True)
        build_dist()
        lint()
        check()
        run_test(release=True, coverage_fail_threshold=75)
        push_pypi()


class TestCommand(Command, object):
    description = 'Runs tests'
    user_options: List = [
        ('release', 'r', 'Run tests as release mode'),
        ('threshold', 't', 'Threshold for coverage failure'),
        ]

    def initialize_options(self):
        self.threshold: int = 75
        self.release: bool = False

    def finalize_options(self):
        pass

    def run(self):
        from scripts.test import run_test
        run_test(release=self.release, coverage_fail_threshold=self.threshold)


class UploadCommand(Command, object):
    description = 'Push package to pypi'
    user_options: List = list()

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from scripts.push import push_pypi
        push_pypi()


class InstallCommandCompletionCommand(Command, object):
    description = 'Install command-completions for shell'
    user_options: List = list()

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        metadata = get_package_metadata()
        install_shell_completion(**metadata)


# ----------------------------------------------------------------------
# Support
# ----------------------------------------------------------------------
files_in_tree = set()
folders_in_tree = set()


def find_data_files(repo_path=None):
    """Captures files that are project specific.

    Args:
        repo_path (str): path to check [default: path of setup.py]

    """
    repo_path = Path(repo_path or Path(__file__).parent).absolute()

    include = ['LICENSE', 'requirements', 'requirements*.txt', 'entrypoints.txt', '*.sql', '*.sql.j2']
    found = sorted(
        str(path.absolute().relative_to(repo_path))
        for path in scan_tree(repo_path)
        if (
            any(path.match(included) for included in include)
            or any(parent.match(included) for parent in path.parents for included in include)
        ))
    return found


def find_packages(repo_path=None, top_package_name=None):
    """Finds packages; replacement for setuptools.find_packages which
    doesn't support PEP 420.

    Lengthy discussion with no resolution:
        https://github.com/pypa/setuptools/issues/97

    Assumptions:
      * Packages are folders
      * Modules are files
      * Namespaces are folders without modules
      * setup.py is for a single package
      * the folder which contains setup.py is _not_ a package itself
      * each package falls under a single tree (e.g. requests, requests.api, etc.)
      * the package is complex enough to have multiple sub folders/modules

    Args:
        repo_path (str): path to check [default: path of setup.py]

    Returns:
        list(list, list, list): Returns packages, modules and namespaces

    """
    repo_path = Path(repo_path or Path(__file__).parent).absolute()
    packages = set()
    modules = set()
    namespaces = set()
    python_artifacts = ['__pycache__']
    repo_artifacts = ['.*']
    install_artifacts = ['*.egg-info']
    build_artifacts = ['dist', 'build']
    test_files = ['tests', 'test', '.tox']
    artifacts = install_artifacts + build_artifacts + repo_artifacts + python_artifacts + test_files
    for path in scan_tree(repo_path):
        path = path.absolute().relative_to(repo_path)

        # Only include paths with .py
        if not path.match('*.py'):
            continue

        # check folders
        if any(p.match(artifact) for artifact in artifacts for p in path.parents):
            continue

        # There should be a valid package at this point
        package_name = str(path.parent).replace(os.path.sep, '.')
        module_name = str(path).replace(os.path.sep, '.').replace(path.suffix, '')
        # TODO: make this actually generic
        if not module_name.startswith(top_package_name):
            continue
        if package_name == '.':
            module_name = f'.{module_name}'
            if module_name == '.setup':
                continue
            modules.add(module_name)
            continue

        modules.add(module_name)
        packages.add(package_name)

    # find namespaces
    for module_name in modules:
        module_path = Path(module_name.replace('.', os.path.sep))
        package_name = str(module_path.parent).replace(os.path.sep, '.')
        if package_name in packages:
            continue
        namespaces.add(package_name)

    return sorted(packages), sorted(modules), sorted(namespaces)


def get_entrypoints(path=None):
    """
    """
    entry_points = {}
    path = path or 'entrypoints.txt'
    if not os.path.exists(path):
        return entry_points

    with open(path) as stream:
        for line in stream:
            if not line.strip():
                continue
            if line.strip().startswith('#'):
                continue
            entry_points.setdefault('console_scripts', []).append(line.strip())
    return entry_points


def get_license(top_path=None):
    """Reads license file and returns"""
    repo_path = top_path or os.path.realpath(os.path.dirname(__file__))
    files = {f.lower(): f for f in os.listdir(repo_path)}
    permutations = itertools.product(['license'], ['', '.txt'])
    files = [os.path.join(repo_path, f) for l, f in files.items() if l in permutations]
    license = ''
    for filepath in files:
        if os.path.exists(filepath):
            with open(filepath, 'r') as stream:
                license = stream.read()
                break
    return license


def get_package_metadata(top_path=None, package_name=None):
    """Find the __metadata__.py file and read it"""
    repo_path = str(Path(top_path or Path(__file__).parent).absolute())
    setup_cfg = Path(repo_path) / 'setup.cfg'
    if package_name is None and setup_cfg.exists():
        for line in setup_cfg.read_text('utf-8').split('\n'):
            if line.startswith("name ="):
                package_name = line.split("name = ")[-1].strip()
                break
    metadata = {}
    prefixes = ('.', '_')
    build_artifacts = ('build', 'test', 'tests')
    paths = [
        path.relative_to(repo_path)
        for path in scan_tree(repo_path)
        if path.stem == '__metadata__'
        ]
    for rel in paths:
        # Don't include the build artifacts
        if not any(parent.stem in build_artifacts for parent in rel.parents):
            # if we have a package name, lets test for it
            if package_name and not any(parent.stem == package_name for parent in rel.parents):
                continue
            d = dict(locals(), **globals())
            exec(rel.read_text('utf-8'), d, d)
            package_metadata = d.get('package_metadata')
            if package_metadata:
                metadata.update(package_metadata.setup)
                break

    requirements, dependency_links = get_package_requirements(top_path=repo_path)
    packages, modules, namespaces = find_packages(top_package_name=metadata['name'])
    # Package Properties
    long_description, long_description_content_type = get_readme()
    metadata['long_description'] = long_description
    metadata['long_description_content_type'] = long_description_content_type
    metadata.setdefault('packages', packages)
    # metadata.setdefault('include_package_data', True)

    # Requirements
    metadata.setdefault('setup_requires', requirements.get('setup') or [])
    metadata.setdefault('install_requires', requirements.get('install') or [])
    metadata.setdefault('tests_require', requirements.get('tests') or requirements.get('test') or [])
    metadata.setdefault('extras_require', requirements.get('extras') or [])
    metadata.setdefault('dependency_links', dependency_links)

    # CLI
    entry_points = get_entrypoints() or {}
    metadata.setdefault('entry_points', entry_points)

    # Packaging
    metadata.setdefault('platforms', ['any'])
    metadata.setdefault('zip_safe', False)

    year = datetime.datetime.now().year
    license = get_license() or 'Copyright {year} - all rights reserved'.format(year=year)
    metadata.setdefault('license', license)

    metadata.setdefault('classifiers', [])
    metadata['classifiers'] = list(metadata['classifiers'])

    # Extra ingestion
    metadata.setdefault('data_files', [('', find_data_files(repo_path))])

    # Add setuptools commands
    metadata.setdefault('cmdclass', get_setup_commands())

    return metadata


def get_package_requirements(top_path=None):
    """Find all of the requirements*.txt files and parse them"""
    repo_path = Path(top_path or Path(__file__).parent).absolute()
    requirements = {'extras': {}}
    dependency_links = set()
    # match on:
    #    requirements.txt
    #    requirements-<name>.txt
    #    requirements_<name>.txt
    #    requirements/<name>.txt
    options = '_-/'
    include_globs = ['requirements*.txt', 'requirements/*.txt']
    paths = [relpath for relpath in scan_tree(repo_path, include=include_globs)]
    for path in paths:
        try:
            path = path.absolute().relative_to(repo_path)
        except Exception:
            raise ValueError(f'\n\nrepo={repo_path}\npath={path}\n\n')
        if path.name == 'requirements.txt' and path.parent.name == '':
            name = 'requirements'
        elif 'requirements' in [p.name for p in path.parents]:
            name = path.name.replace(path.suffix, '')
        elif 'requirements' in path.name:
            name = path.name.replace('requirements', '').lstrip(options)
        else:
            raise Exception(f'Could not find requirements using {path}')
        reqs_, deps = parse_requirements(str(path.absolute()))
        dependency_links.update(deps)
        if name in ['requirements', 'install', '']:
            requirements['install'] = reqs_
        elif name in ['test', 'tests']:
            requirements['tests'] = reqs_
            requirements['extras']['tests'] = reqs_
        elif name in ['setup']:
            requirements['setup'] = reqs_
        else:
            requirements['extras'][name] = reqs_

    all_reqs = set()
    dev_reqs = set()
    for name, req_list in requirements.items():
        if name in ['install']:
            all_reqs.update(req_list)
        elif name in ['extras']:
            for subname, reqs in req_list.items():
                all_reqs.update(reqs)
                dev_reqs.update(reqs)
        else:
            all_reqs.update(req_list)
            dev_reqs.update(req_list)

    requirements['extras']['dev'] = list(sorted(dev_reqs))
    requirements['extras']['all'] = list(sorted(all_reqs))
    return requirements, list(sorted(dependency_links))


def get_readme(top_path: Optional[Path] = None) -> Tuple[str, str]:
    """Read the readme for the repo"""
    readme = ''
    path = Path(__file__).parent if top_path is None else top_path
    paths = {p.relative_to(path) for p in path.glob('*')}
    permutations = [
        ''.join(permutation)
        for permutation in itertools.product(['readme'], ['', '.md', '.rst', '.txt'])
        ]
    filepaths = {p for p in paths if p.name.lower() in permutations}
    content_type = 'text/x-rst'  # see: https://packaging.python.org/specifications/core-metadata/#description-content-type
    # Grabs the first one found
    for filepath in filepaths:
        readme = filepath.read_text('utf-8')
        if filepath.name.endswith('.txt'):
            content_type = 'text/plain'
        elif filepath.name.endswith('.md'):
            content_type = 'text/markdown'
        elif filepath.name.endswith('.rst'):
            content_type = 'text/x-rst'
        else:
            content_type = 'text/plain'
        break
    # See: https://github.com/pypa/setuptools/issues/1390
    readme = re.sub(pattern='\n\n+', repl='\n\n', string=readme)
    return readme, content_type


def get_setup_commands():
    """Returns setup command class list"""
    commands = {
        'build_image': BuildImageCommand,
        'check': CheckCommand,
        'cli': InstallCommandCompletionCommand,
        'clean': CleanCommand,
        'dist': DistCommand,
        'lint': LintCommand,
        'prerelease': PrereleaseCommand,
        'release': ReleaseCommand,
        'test': TestCommand,
        'upload': UploadCommand,
        }
    return commands


def install_shell_completion(**metadata):
    for cep in metadata['entry_points']['console_scripts']:
        name, _ = cep.split('=', 1)
        name = name.strip()
        cmd = f'{name} shell-completion install'
        subprocess.run(cmd, shell=True, check=False)  # some commands can't complete


def parse_requirements(path):
    template = '{name}{spec}'
    requirements = set()
    dependency_links = set()
    for requirement in req.parse_requirements(path, session="somesession"):
        if requirement.markers is not None and not requirement.markers.evaluate():
            continue

        name = requirement.name
        spec = str(requirement.req.specifier) if len(str(requirement.req.specifier)) else ''
        req_ = template.format(name=name, spec=spec)
        if req_:
            requirements.add(req_)

        link = str(requirement.link) if requirement.link else ''
        if link:
            dependency_links.add(link)

        # TODO: What do we do with these?
        if requirement.options:
            pass

    return list(sorted(requirements)), list(sorted(dependency_links))


def scan_tree(top_path=None, exclude=None, include=None):
    """Finds files in tree

    * Order is random
    * Folders which start with . and _ are excluded unless excluded is used (e.g. [])
    * This list is memoized

    Args:
        top_path (str): top of folder to search

    Yields:
        str: paths as found
    """
    repo_path = Path(top_path or Path(__file__).parent).absolute()
    if not files_in_tree:
        for root, folders, files in os.walk(repo_path):
            rel = Path(root).relative_to(repo_path)
            # Control traversal
            folders[:] = [f for f in folders if f not in ['.git']]
            folders_in_tree.update(folders)
            # Yield files
            for filename in files:
                relpath = rel.joinpath(filename)
                if relpath not in files_in_tree:
                    files_in_tree.add(relpath)
                    if include is not None:
                        if any(relpath.match(inc) for inc in include):
                            yield repo_path / relpath
                    else:
                        yield repo_path / relpath
    else:
        for relpath in files_in_tree:
            if include:
                if any(relpath.match(inc) for inc in include):
                    yield repo_path / relpath
            else:
                yield repo_path /relpath


if __name__ == '__main__':
    main()



from dataclasses import dataclass

import pytest


@dataclass
class StripEscapeData:
    input: str = ''
    output: str = ''


@pytest.mark.parametrize('test_data', [
    StripEscapeData('\x1b[31ma\x1b[0m', 'a'),
    StripEscapeData('b', 'b'),
    StripEscapeData('\033[32mc\033[0m', 'c'),
    StripEscapeData('', ''),
    StripEscapeData('\033[33m\033[0m', ''),
    StripEscapeData('\x1b[38;5;245ma\x1b[39m', 'a'),
    StripEscapeData(
        input='''{"data": "available=\u001b[31mNone\u001b[0m download=\u001b[35mNone\u001b[0m clean=\u001b[33mNone\u001b[0m verbose=\u001b[32m1\u001b[0m", "timestamp": "2019-02-22 22:57:04.459103", "available": null, "download": null, "clean": null, "verbose": 1}''',
        output='''{"data": "available=None download=None clean=None verbose=1", "timestamp": "2019-02-22 22:57:04.459103", "available": null, "download": null, "clean": null, "verbose": 1}''',
        ),
    StripEscapeData(
        input='''{"data": "Remote: \u001b[32mrecommendations/audience_similarity/v4/panel=aus/external/start_yyyymmdd=20150801/end_yyyymmdd=20181129/series_map_yyyymmdd=20181129/_SUCCESS\u001b[0m", "timestamp": "2019-02-25 19:46:46.988875"}''',
        output='''{"data": "Remote: recommendations/audience_similarity/v4/panel=aus/external/start_yyyymmdd=20150801/end_yyyymmdd=20181129/series_map_yyyymmdd=20181129/_SUCCESS", "timestamp": "2019-02-25 19:46:46.988875"}''',
        )
    ])
def test_strip_escape(test_data):
    from ..message import strip_escape

    value = strip_escape(test_data.input)
    assert value == test_data.output



from dataclasses import asdict, dataclass
from pathlib import Path
from typing import Any

import pytest


@dataclass
class BeautifyData:
    message: Any
    expected: str = ''
    indent: int = 0
    lexer: Any = None


@pytest.mark.parametrize('test_data', [
    BeautifyData('a', expected='a'),  # expected='\x1b[38;5;247ma\x1b[39m'),
    BeautifyData('ls -lashtr $HOME', lexer='bash', expected='ls -lashtr \x1b[38;2;25;23;124m$HOME\x1b[39m'),
    BeautifyData('SELECT * FROM some_table;', expected='\x1b[38;2;0;128;0;01mSELECT\x1b[39;00m \x1b[38;2;102;102;102m*\x1b[39m \x1b[38;2;0;128;0;01mFROM\x1b[39;00m some_table;'),
    BeautifyData('SELECT * FROM some_table;', lexer='postgres', expected='\x1b[38;2;0;128;0;01mSELECT\x1b[39;00m \x1b[38;2;102;102;102m*\x1b[39m \x1b[38;2;0;128;0;01mFROM\x1b[39;00m some_table;'),
    BeautifyData(Path('.'), expected='.'),
    ])
def test_beautify(test_data):
    from ..formatting import beautify

    input_params = asdict(test_data)
    expected = input_params.pop('expected')
    result = beautify(**input_params)
    assert result == expected, f'{result} did not match {expected}, {(result,)}'
